// vim: set tw=80 ts=2 sw=2 sts=2 :
import { Component, Inject, OnInit } from '@angular/core';
import { MdDialog } from '@angular/material';
import { Observable } from 'rxjs/Rx';

import { Logger } from '@nsalaun/ng-logger';
import { ToastrService } from 'ngx-toastr';

import { Account } from './account';
import { AccountBalances } from './accountBalances';
import { AccountService } from './account.service';
import { AccountEditModalComponent } from './accountEditModal.component';

@Component({
  selector: 'account-list',
  template: `
<div class="row">
  <div class="col s12">
    <table class="bordered highlight responsive-table">
      <thead>
        <tr>
          <th>Nom du compte</th>
          <th>Solde courant</th>
          <th>Solde pointé</th>
          <th>Découvert autorisé</th>
          <th>Actions</th>
        </tr>
      </thead>

      <tbody>
        <tr>
          <td colspan="5">
            <button md-raised-button color="primary" (click)="add()">
              Ajouter
            </button>
          </td>
        </tr>

        <tr *ngFor="let account of accounts"
          [account-row]="account" (needsReload)="load()">
        </tr>
      </tbody>
    </table>
  </div>
</div>
    `,
})
export class AccountListComponent implements OnInit {
  accounts: Account[];

  constructor(
    private accountService: AccountService,
    private toastrService: ToastrService,
    private logger: Logger,
    private mdDialog: MdDialog,
  ) {
  }

  ngOnInit() {
    // Load accounts.
    this.load();
  }

  load() {
    this.logger.log("Load accounts.");
    this.accountService.query().subscribe(accounts => {
      this.accounts = accounts;
    });
  };

  /*
   * Add an empty account.
   */
  add() {
    let dialogRef = this.mdDialog.open(AccountEditModalComponent, {
      data: {
        account: new Account(),
      }
    });

    dialogRef.afterClosed().subscribe((account: Account) => {
      if(account) {
        this.logger.log("Modal closed => save account", account);
        this.save(account);
      } else {
        this.logger.log("Modal dismissed");
      }
    }, (reason) => function(reason) {
    });
  };

  /*
   * Save account.
   */
  save(account) {
    this.accountService.create(account).subscribe(account => {
      this.toastrService.success('Account #' + account.id + ' saved.');

      this.load();
    }, result => {
      this.logger.error('Error while saving account', account, result);

      this.toastrService.error(
        'Error while saving account: ' + result.message
      );
    });
  };
};
